package edu.swe622.fers.service;

import java.io.Serializable;
import java.rmi.RemoteException;

import org.apache.axis2.AxisFault;

import edu.swe622.fers.model.Location;
import edu.swe622.fers.model.Sensor;

public class VehicleItem implements Serializable
{
    public static final int AT_BASE = 0;
    public static final int DISPATCHED = 1;
    public static final int DISPATCHED_ON_WAY = 2;
    public static final int DISPATCHED_AT_LOCATION = 3;
    public static final int DISPATCHED_EXT_FIRE = 4;
    public static final int DISPATCHED_MISSION_COMPLETE = 5;
    public static final int RETURNING_TO_BASE = 6;
                    
    
    private String vehicleID = null;
    private String vehicleServiceUrl = null;
    private Location currentLocation;
    private Location previousLocation = null;
    private int myVehicleStatus = AT_BASE;


    // Default Constructor
    public VehicleItem () {}
    
    // Constructor
    public VehicleItem (String newVehicleID, String newVehicleServiceUrl, Location newLocation) 
    {
        if ( newVehicleID == null ) throw new NullPointerException("Vehicle.Vehicle, vehicleID is null.");
        if ( newVehicleServiceUrl == null ) throw new NullPointerException("Vehicle.Vehicle, newVehicleHandle is null.");
        if ( newLocation == null ) throw new NullPointerException("Vehicle.Vehicle, newLocation is null.");
        
        this.vehicleServiceUrl = newVehicleServiceUrl;
        
        this.vehicleID = newVehicleID;
        this.setCurrentLocation(newLocation);
        this.setVehicleServiceUrl(newVehicleServiceUrl);
    }
    
    // Public Methods
    
    public String getVehicleID()
    {
        return this.vehicleID;
    }
    
    public Location getCurrentLocation () 
    {
        return this.currentLocation; //  return a defencive copy;
    }
    
    public Location getPreviousLocation () 
    {
        return this.previousLocation; //  return a defencive copy;
    }
    

    public void setCurrentLocation (Location newLocation) 
    {
        this.previousLocation = this.currentLocation;
        this.currentLocation = newLocation.clone();  // make defencive copy.
    }

    public int getStatus () 
    {
        return this.myVehicleStatus;
    }

    public void setStatus (int newStatus) 
    {
        if (newStatus < AT_BASE || newStatus > RETURNING_TO_BASE) 
            throw new IllegalArgumentException("Vehicle.setStatus, newStatus = " + newStatus);
        this.myVehicleStatus = newStatus;
    }

    public String getVehicleHandle () 
    {
        return this.vehicleServiceUrl;
    }
    
    public void setVehicleHandle (String newVehicleServiceUrl)
    {
        this.vehicleServiceUrl = newVehicleServiceUrl;
    }

    private void setVehicleServiceUrl (String newVehicleServiceUrl) 
    {
        this.vehicleServiceUrl = newVehicleServiceUrl;
    }

    public boolean isDispatched () 
    {
        if ( this.myVehicleStatus == DISPATCHED ) return true;
        if ( this.myVehicleStatus == DISPATCHED_ON_WAY ) return true;
        if ( this.myVehicleStatus == DISPATCHED_AT_LOCATION ) return true;
        if ( this.myVehicleStatus == DISPATCHED_MISSION_COMPLETE ) return true;
        if ( this.myVehicleStatus == DISPATCHED_EXT_FIRE ) return true;
        
        return false;
    }

    
    // Methods that call vehicle services:
    
    // dispatch to FireFitingEngine service
    public void dispatch (Sensor newSensor, Location newSchoolLocation, boolean newIsPrimary, String newServiceContext) 
    {
        System.out.println("VehicleItem.dispatch: newSensor: " + newSensor.toString() + 
                ", newSchoolLocation: " + newSchoolLocation.toString() + 
                ", newIsPrimary: " + newIsPrimary ); // Logging
        try 
        {			
            // Get Service Stub for this FFEngine.
            FireFightingEngineServiceStub ffEngineStub = new FireFightingEngineServiceStub(vehicleServiceUrl);

            // Build dispatch message
            FireFightingEngineServiceStub.Dispatch ffeDispatch = new FireFightingEngineServiceStub.Dispatch();

            // School Location
            ffeDispatch.setDst( new FireFightingEngineServiceStub.Location() );
            ffeDispatch.getDst().setCordX( newSchoolLocation.getCordX() );
            ffeDispatch.getDst().setCordY( newSchoolLocation.getCordY() );

            // Is primary
            ffeDispatch.setIsPrimary(newIsPrimary);

            // Smoke Detecktor Sensor
            ffeDispatch.setSmokeDetector( new FireFightingEngineServiceStub.Sensor() );
            ffeDispatch.getSmokeDetector().setLocation( new FireFightingEngineServiceStub.Location() );
            ffeDispatch.getSmokeDetector().getLocation().setCordX( newSensor.getLocation().getCordX() );
            ffeDispatch.getSmokeDetector().getLocation().setCordY( newSensor.getLocation().getCordY() );
            ffeDispatch.getSmokeDetector().setSensorId( newSensor.getSensorId() );
            ffeDispatch.getSmokeDetector().setSchoolId( newSensor.getSchoolId() );
            ffeDispatch.getSmokeDetector().setStatus( newSensor.getStatus() );
            ffeDispatch.getSmokeDetector().setType( newSensor.getType() );
            ffeDispatch.getSmokeDetector().setNearestFireStation( newSensor.getNearestFireStation() );
            
            // Set Organization with service ID for jUDDI.  Same as dispatch service serviceContext.
            ffeDispatch.setOrg( new FireFightingEngineServiceStub.Organization() );
            ffeDispatch.getOrg().setUri(newServiceContext);
            
            // Send dispatch message to FFEngine Service.
            ffEngineStub.dispatch( ffeDispatch );	
        } 
        catch (AxisFault e) { e.printStackTrace(); }
        catch (RemoteException e) { e.printStackTrace(); }
        
        this.setStatus(DISPATCHED);
    }
    
        
    @Override
	public String toString() {
		// TODO Auto-generated method stub
		return "ID:" + vehicleID + " ServiceURL:" + vehicleServiceUrl + " Current: " + currentLocation;
	}

	// dispatch to Ambulance service
    public void dispatch (Location newSchoolLocation, String newServiceContext) 
    {
        System.out.println("VehicleItem.dispatch:" + 
                ", newSchoolLocation: " + newSchoolLocation.toString() + 
                ", newServiceContext: " + newServiceContext ); // Logging
        try 
        {			
        	System.out.println(this);
            // Get Service Stub for this Ambulance.
            AmbulanceServiceStub ambulanceStub = new AmbulanceServiceStub(vehicleServiceUrl);

            // Build dispatch message
            AmbulanceServiceStub.Dispatch ambulanceDispatch = new AmbulanceServiceStub.Dispatch();

            // School Location
            ambulanceDispatch.setDst( new AmbulanceServiceStub.Location() );
            ambulanceDispatch.getDst().setCordX( newSchoolLocation.getCordX() );
            ambulanceDispatch.getDst().setCordY( newSchoolLocation.getCordY() );          
                        
            // Set Organization with dispatch service ID for jUDDI.  Same as dispatch service serviceContext.
            ambulanceDispatch.setOrg( new AmbulanceServiceStub.Organization() );
            ambulanceDispatch.getOrg().setUri(newServiceContext);

            // Send dispatch message to Ambulance Service.
            ambulanceStub.dispatch( ambulanceDispatch );	
        } 
        catch (AxisFault e) { e.printStackTrace(); }
        catch (RemoteException e) { e.printStackTrace(); }
        
        this.setStatus(DISPATCHED);
    }
    
    public void addSensorToPrimaryVehicle( Sensor newSensor )
    {
        System.out.println("VehicleItem.addSensorToPrimaryVehicle: vehicleID: " + this.vehicleID +  
                ", newSensor: " + newSensor.toString()); // Logging
        
        try 
        {			
            // Get Service Stub for this FFEngine.
            FireFightingEngineServiceStub ffEngineStub = new FireFightingEngineServiceStub(vehicleServiceUrl);

            // Build dispatch message
            FireFightingEngineServiceStub.AddSensor ffeAddSensor = new FireFightingEngineServiceStub.AddSensor();

            // Smoke Detecktor Sensor
            ffeAddSensor.setNewSD( new FireFightingEngineServiceStub.Sensor() );
            ffeAddSensor.getNewSD().setLocation( new FireFightingEngineServiceStub.Location() );
            ffeAddSensor.getNewSD().getLocation().setCordX( newSensor.getLocation().getCordX() );
            ffeAddSensor.getNewSD().getLocation().setCordY( newSensor.getLocation().getCordY() );
            ffeAddSensor.getNewSD().setSensorId( newSensor.getSensorId() );
            ffeAddSensor.getNewSD().setSchoolId( newSensor.getSchoolId() );
            ffeAddSensor.getNewSD().setStatus( newSensor.getStatus() );
            ffeAddSensor.getNewSD().setType( newSensor.getType() );
            ffeAddSensor.getNewSD().setNearestFireStation( newSensor.getNearestFireStation() );
            
            // Send dispatch message to FFEngine Service.
            ffEngineStub.addSensor( ffeAddSensor );	
        } 
        catch (AxisFault e) { e.printStackTrace(); }
        catch (RemoteException e) { e.printStackTrace(); }
    }
    
    
    public void getLocationUpdateFromEngine()
    {     
        try 
        {		                      
            // Get Service Stub for this FFEngine.
            FireFightingEngineServiceStub ffEngineStub = new FireFightingEngineServiceStub(vehicleServiceUrl);
            
            Location currentVehicleLocation = convertToLocation( ffEngineStub.getCurrentLocation().get_return() );

            this.setCurrentLocation(currentVehicleLocation);
        }
        catch (AxisFault e) { e.printStackTrace(); } 
        catch (RemoteException e) { e.printStackTrace(); } 
     
        System.out.println("VehicleItem.getLocationUpdateFromVehicle: vehicleID: " + this.vehicleID + 
                ", new currentLocation  " + this.currentLocation.toString() ); // Logging
    }
    
    public void getLocationUpdateFromAmbulance()
    {     
        try 
        {		                      
            // Get Service Stub for this FFEngine.
            AmbulanceServiceStub ambulanceStub = new AmbulanceServiceStub(vehicleServiceUrl);
            
            Location currentVehicleLocation = convertToLocation( ambulanceStub.getCurrentLocation().get_return() );

            this.setCurrentLocation(currentVehicleLocation);
        }
        catch (AxisFault e) { e.printStackTrace(); } 
        catch (RemoteException e) { e.printStackTrace(); } 
     
        System.out.println("VehicleItem.getLocationUpdateFromVehicle: vehicleID: " + this.vehicleID + 
                ", new currentLocation  " + this.currentLocation.toString() ); // Logging
    }
        
    private Location convertToLocation(AmbulanceServiceStub.Location tempLoc)
    {
        Location location = new Location();
        location.setCordX(tempLoc.getCordX());
        location.setCordY(tempLoc.getCordY());
        return location;
    }
        
    private Location convertToLocation(FireFightingEngineServiceStub.Location tempLoc)
    {
        Location location = new Location();
        location.setCordX(tempLoc.getCordX());
        location.setCordY(tempLoc.getCordY());
        return location;
    }
    	
    
    public void recallEngine () 
    {
        System.out.println("VehicleItem.recall: vehicleID: " + this.vehicleID ); // Logging
        
        try
        {			
            // Get Service Stub for this FFEngine.
            FireFightingEngineServiceStub ffEngineStub = new FireFightingEngineServiceStub(vehicleServiceUrl);
            
            // Send send recall to FFEngine Service.
            ffEngineStub.doReturn();	
        } 
        catch (AxisFault e) { e.printStackTrace(); }
        catch (RemoteException e) { e.printStackTrace(); }     
    }
    
    public void recallAmbulance () 
    {
        System.out.println("VehicleItem.recallAmbulance: vehicleID: " + this.vehicleID ); // Logging
        
        try
        {			
            // Get Service Stub for this FFEngine.
            AmbulanceServiceStub ambulanceStub = new AmbulanceServiceStub(vehicleServiceUrl);
            
            // Send send recall to FFEngine Service.
            ambulanceStub.doReturn();	
        } 
        catch (AxisFault e) { e.printStackTrace(); }
        catch (RemoteException e) { e.printStackTrace(); }     
    }

    
    
    // Standard Object Methods
    
    
    public boolean equals (Object otherObject) 
    {
        if ( otherObject == this ) return true;
        if ( otherObject == null ) return false;
        if ( otherObject.getClass() != this.getClass() ) return false;
        
        VehicleItem otherVehicle = (VehicleItem) otherObject;
        
        if ( this.getVehicleID().equals(otherVehicle.getVehicleID()) ) return true;
        
        return false;
    }

    public int hashCode () 
    {
        return this.getVehicleID().hashCode();
    }
    
    public boolean repOK()
    {
        
        if ( this.currentLocation == null ) return false;
        if ( this.vehicleServiceUrl == null ) return false;
        if ( this.vehicleServiceUrl.equals("")) return false; // need to improve

        if ( this.myVehicleStatus == AT_BASE ) return true;
        if ( this.myVehicleStatus == DISPATCHED )  return true;
        if ( this.myVehicleStatus == DISPATCHED_ON_WAY )  return true;
        if ( this.myVehicleStatus == DISPATCHED_AT_LOCATION ) return true;
        if ( this.myVehicleStatus == DISPATCHED_EXT_FIRE )  return true;
        if ( this.myVehicleStatus == DISPATCHED_MISSION_COMPLETE )  return true;
        if ( this.myVehicleStatus == RETURNING_TO_BASE )  return true;
        
        return false;
    }

}

