package edu.swe622.fers.model;

/**
 * @author Naeem
 */
public class LocationXY extends Location{	
	/**
	 * This ID is required for Serialization
	 */
	private static final long serialVersionUID = 9027921905289302100L;
	
	public LocationXY(){}
	
	public LocationXY(double x, double y) {
		super();
		this.cordX = x;
		this.cordY = y;
	}	
		
	/**
	 * @param other
	 * @deprecated
	 */
	public LocationXY(LocationXY other) {
		super();
		this.cordX = other.cordX;
		this.cordY = other.cordY;
	}

	/**
	 * Computes the manhattan distance between to locations
	 * @param location2 second location
	 * @return the manhattan distance between them
	 * @throws LocationMismatch LocationMismatch the system must provide unify location types
	 */
	public double howFarIs(Location location2) throws LocationMismatch {
		if(!(location2 instanceof LocationXY))
			throw new LocationMismatch();
		
		LocationXY loc2 = (LocationXY) location2;
		
		return Math.abs(cordX - loc2.cordX) + Math.abs(cordY - loc2.cordY);
	}

	/**
	 * Moves source one step (speed) toward the destination. First X then Y.
	 * @param destination The place where we should be 
	 * @param distancePerSecond The speed of the vehicle (not zero)
	 * @return If we have received to destination
	 * @throws InfiniteLoopException If the speedis equal to zero we will never be there
	 * @throws LocationMismatch LocationMismatch the system must provide unify location types
	 */
	public boolean moveToward(Location destination, double distancePerSecond) throws InfiniteLoopException, LocationMismatch {
		if(!(destination instanceof LocationXY))
			throw new LocationMismatch();
		
		LocationXY dst = (LocationXY) destination;

		if(distancePerSecond <= 0)
			throw new InfiniteLoopException();
		
		if(equals(dst))
			return true;
		
		//How many steps we have toward the destination
		double step = distancePerSecond;		
		
		//How far the destination is
		double howFar = Math.abs(dst.cordX - cordX);
		
		if(howFar >= step) {
			cordX += step * getDirection(cordX, dst.cordX); //Going toward the destination
			return equals(dst);
		}
		
		//else
		if(howFar < step) {
			cordX = dst.cordX;
			
			step -= howFar;
			howFar = Math.abs(dst.cordY - cordY);
			
			if(howFar >= step) {
				cordY += step * getDirection(cordY, dst.cordY);
				return equals(dst);
			}
			
			//else
			if(howFar < step) {
				cordY = dst.cordY;
				return equals(dst); //equals(dst) = true
			}
		}
		
		throw new InfiniteLoopException();
	}
			
	@Override
	public LocationXY clone() {		
		return new LocationXY(cordX, cordY);
	}

	/**
	 * Tells if source should be increased to approach the destination or vice versa. This method makes sense when source != destination.
	 * @param from source
	 * @param to destinatio
	 * @return 1: increase, -1: decreas, never 0: never called on equal ones
	 */
	protected int getDirection(double from, double to) {
		if(from < to)
			return 1;
		
		return -1;
	}
}
